from .tools.environment import create_file, create_folders, check_jars
from .config import logs_folder, datalake, datamart


def set_environment_utils():
    create_folders([logs_folder, datalake, datamart])
    env_content = (
        'ORIGIN_DB={"host": "host", "port": 1433, "user": "user", "pwd": "password", "db_name": "some_db"}\n'
        'DESTINY_DB={"host": "host", "port": 1433, "user": "user", "pwd": "password", "db_name": "some_db"}\n'
    )
    create_file(".env", env_content)
    create_file(
        "arquee.txt",
        "Here you can specify the tables for create the datalake, delete this line :)\n",
    )
    check_jars()


def otro_test():
    pass


if __name__ == "__main__":
    set_environment_utils()



from os.path import exists, join
from .config import datalake, datamart
from .tools.db import get_db_url
from pyspark.sql import DataFrame
from pyspark.sql import SparkSession
from dotenv import load_dotenv
from pyspark.sql.functions import initcap, trim
from os import listdir

load_dotenv()


def create_datalake(file_name: str = "arquee.txt", clean: bool = True):
    origin = get_db_url("ORIGIN_DB")

    if not exists(datalake):
        raise FileNotFoundError("Folder datalake not found, run command Arquee")

    if exists(file_name):
        spark = get_spark_session()
        with open(file_name, "r") as file:
            tables = set(table.upper().strip() for table in file.readlines())

    for table in tables:
        destiny_path = join(datalake, table)
        df: DataFrame = spark.read.jdbc(origin, table)

        if clean:
            for column in df.columns:
                df = df.withColumn(column, trim(initcap(column)))

            df = df.fillna("No encontrado")
            df = df.replace("", value="No encontrado")

        df.write.parquet(destiny_path, mode="overwrite")


spark = None


def get_spark_session() -> SparkSession:
    global spark
    if not spark:
        spark = (
            SparkSession.builder.appName("ETL")
            .master("local[*]")
            .config("spark.executor.memory", "4g")
            .config("spark.sql.parquet.int96RebaseModeInWrite", "CORRECTED")
            .getOrCreate()
        )
        spark.sparkContext.setLogLevel("ERROR")
    return spark


def get_dataframe(table_name: str, standart_location: bool = True):
    spark: SparkSession = get_spark_session()

    if standart_location:
        folder_path = join(datalake, table_name.upper().strip())
    else:
        folder_path = join(datamart, table_name.upper().strip())

    if not exists(folder_path):
        raise FileNotFoundError("Folder not found, check datalake folder")
    df: DataFrame = spark.read.parquet(folder_path)
    return df


def save_dataframe(df: DataFrame, table_name: str):
    folder_path = join(datamart, table_name.upper().strip())
    df.write.parquet(folder_path, "overwrite")


def save_datamart():
    destiny = get_db_url("DESTINY_DB")
    dm_tables = listdir(datamart)
    print(dm_tables)
    for table in dm_tables:
        df = get_dataframe(table, standart_location=False)
        df.write.jdbc(destiny, table, mode="overwrite")


if __name__ == "__main__":
    # create_datalake(clean=False)
    save_datamart()



import json
from json.decoder import JSONDecodeError
from os import getenv
from dotenv import load_dotenv

load_dotenv()


def get_db_url(variable_name: str):
    expected_keys = ["host", "port", "user", "pwd", "db_name"]
    try:
        db_variable = getenv(variable_name)
        values = json.loads(db_variable)
        missing_keys = [key for key in expected_keys if key not in list(values.keys())]
        
        if missing_keys:
            raise ValueError(missing_keys)

        host = values["host"]
        db_name = values["db_name"]
        user = values["user"]
        pwd = values["pwd"]
        port = values["port"]

        url = f"jdbc:sqlserver://{host}:{port};databaseName={db_name};user={user};password={pwd};encrypt=false;"

        return url

    except ValueError as e:
        print(f"Missing {list(e.args)[0]} keys on JSON object")

    except JSONDecodeError:
        print(f"Error reading {variable_name} variable, variable must be a JSON object")



from dotenv import load_dotenv
from os import makedirs
from os.path import exists, join
from shutil import move
import urllib.request
from urllib.error import URLError
from platform import system

load_dotenv()


def create_folders(folders):
    for folder in folders:
        try:
            makedirs(folder, exist_ok=True)
        except Exception as e:
            print(f"Error creating folder {folder}: {e}")


def create_file(filename, content):
    if not exists(filename):
        try:
            with open(filename, "w") as file:
                file.write(content)
            print(f"File {filename} created successfully")
        except Exception as e:
            print(f"Error creating file {filename}: {e}")
    else:
        print(f"{filename} already exist, skipping creation")


def download_mssql_driver(jars_folder):
    try:
        file = "mssql-jdbc-12.8.1.jre11.jar"
        complete_jars_path = join(jars_folder, file)
        if exists(complete_jars_path):
            print("JDBC mssql driver is alredy downloaded")
            return
        url = "https://repo1.maven.org/maven2/com/microsoft/sqlserver/mssql-jdbc/12.8.1.jre11/mssql-jdbc-12.8.1.jre11.jar"
        urllib.request.urlretrieve(url, file)
        move(file, jars_folder)
        print("Download of JDBC mssql driver is done")
    except URLError:
        print(
            "Error downloading the MSSQL JDBC driver. Please check your internet connection."
        )


def check_jars():
    platform = system().lower()
    if platform == "windows":
        jars_folder = join(".venv", "Lib", "site-packages", "pyspark", "jars")
    else:
        jars_folder = join(
            ".venv", "lib", "python3.9", "site-packages", "pyspark", "jars"
        )
    download_mssql_driver(jars_folder)
